from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.data_rows import DataRows
from ...models.http_validation_error import HTTPValidationError
from ...models.split import Split
from ...models.summary_request import SummaryRequest
from ...types import UNSET, Response, Unset


def _get_kwargs(
    project_id: str, run_id: str, split: Split, *, body: SummaryRequest, inference_name: Union[Unset, str] = ""
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    params: dict[str, Any] = {}

    params["inference_name"] = inference_name

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": f"/projects/{project_id}/runs/{run_id}/split/{split}/insights/rows",
        "params": params,
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[DataRows, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = DataRows.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[DataRows, HTTPValidationError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_id: str,
    run_id: str,
    split: Split,
    *,
    client: AuthenticatedClient,
    body: SummaryRequest,
    inference_name: Union[Unset, str] = "",
) -> Response[Union[DataRows, HTTPValidationError]]:
    """Get Data Rows

     Returns rows of data for the run/split with optional filters applied.

    Also responds if there is a next page available (more rows)
    [MLTC] - A task must be provided
    [OD] - A default map_threshold is provided at 0.5. This can be changed in the body
        of the request, and will result in different error values for each returned row

    Args:
        project_id (str):
        run_id (str):
        split (Split):
        inference_name (Union[Unset, str]):  Default: ''.
        body (SummaryRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[DataRows, HTTPValidationError]]
    """

    kwargs = _get_kwargs(project_id=project_id, run_id=run_id, split=split, body=body, inference_name=inference_name)

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    project_id: str,
    run_id: str,
    split: Split,
    *,
    client: AuthenticatedClient,
    body: SummaryRequest,
    inference_name: Union[Unset, str] = "",
) -> Optional[Union[DataRows, HTTPValidationError]]:
    """Get Data Rows

     Returns rows of data for the run/split with optional filters applied.

    Also responds if there is a next page available (more rows)
    [MLTC] - A task must be provided
    [OD] - A default map_threshold is provided at 0.5. This can be changed in the body
        of the request, and will result in different error values for each returned row

    Args:
        project_id (str):
        run_id (str):
        split (Split):
        inference_name (Union[Unset, str]):  Default: ''.
        body (SummaryRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[DataRows, HTTPValidationError]
    """

    return sync_detailed(
        project_id=project_id, run_id=run_id, split=split, client=client, body=body, inference_name=inference_name
    ).parsed


async def asyncio_detailed(
    project_id: str,
    run_id: str,
    split: Split,
    *,
    client: AuthenticatedClient,
    body: SummaryRequest,
    inference_name: Union[Unset, str] = "",
) -> Response[Union[DataRows, HTTPValidationError]]:
    """Get Data Rows

     Returns rows of data for the run/split with optional filters applied.

    Also responds if there is a next page available (more rows)
    [MLTC] - A task must be provided
    [OD] - A default map_threshold is provided at 0.5. This can be changed in the body
        of the request, and will result in different error values for each returned row

    Args:
        project_id (str):
        run_id (str):
        split (Split):
        inference_name (Union[Unset, str]):  Default: ''.
        body (SummaryRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[DataRows, HTTPValidationError]]
    """

    kwargs = _get_kwargs(project_id=project_id, run_id=run_id, split=split, body=body, inference_name=inference_name)

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_id: str,
    run_id: str,
    split: Split,
    *,
    client: AuthenticatedClient,
    body: SummaryRequest,
    inference_name: Union[Unset, str] = "",
) -> Optional[Union[DataRows, HTTPValidationError]]:
    """Get Data Rows

     Returns rows of data for the run/split with optional filters applied.

    Also responds if there is a next page available (more rows)
    [MLTC] - A task must be provided
    [OD] - A default map_threshold is provided at 0.5. This can be changed in the body
        of the request, and will result in different error values for each returned row

    Args:
        project_id (str):
        run_id (str):
        split (Split):
        inference_name (Union[Unset, str]):  Default: ''.
        body (SummaryRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[DataRows, HTTPValidationError]
    """

    return (
        await asyncio_detailed(
            project_id=project_id, run_id=run_id, split=split, client=client, body=body, inference_name=inference_name
        )
    ).parsed



from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="MetricCritiqueColumn")


@_attrs_define
class MetricCritiqueColumn:
    """
    Attributes:
        col_name (str):
        metric_critique_computing (bool):
    """

    col_name: str
    metric_critique_computing: bool
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        col_name = self.col_name

        metric_critique_computing = self.metric_critique_computing

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"col_name": col_name, "metric_critique_computing": metric_critique_computing})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        col_name = d.pop("col_name")

        metric_critique_computing = d.pop("metric_critique_computing")

        metric_critique_column = cls(col_name=col_name, metric_critique_computing=metric_critique_computing)

        metric_critique_column.additional_properties = d
        return metric_critique_column

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Literal, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="TagsConstraints")


@_attrs_define
class TagsConstraints:
    """
    Attributes:
        feedback_type (Literal['tags']):
        tags (list[str]):
        allow_other (Union[Unset, bool]):  Default: False.
    """

    feedback_type: Literal["tags"]
    tags: list[str]
    allow_other: Union[Unset, bool] = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        feedback_type = self.feedback_type

        tags = self.tags

        allow_other = self.allow_other

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"feedback_type": feedback_type, "tags": tags})
        if allow_other is not UNSET:
            field_dict["allow_other"] = allow_other

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        feedback_type = cast(Literal["tags"], d.pop("feedback_type"))
        if feedback_type != "tags":
            raise ValueError(f"feedback_type must match const 'tags', got '{feedback_type}'")

        tags = cast(list[str], d.pop("tags"))

        allow_other = d.pop("allow_other", UNSET)

        tags_constraints = cls(feedback_type=feedback_type, tags=tags, allow_other=allow_other)

        tags_constraints.additional_properties = d
        return tags_constraints

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.get_presigned_url_response import GetPresignedUrlResponse
from ...models.http_validation_error import HTTPValidationError
from ...models.method import Method
from ...types import UNSET, Response


def _get_kwargs(*, api_url: str, method: Method, bucket_name: str, object_name: str, project_id: str) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["api_url"] = api_url

    json_method = method.value
    params["method"] = json_method

    params["bucket_name"] = bucket_name

    params["object_name"] = object_name

    params["project_id"] = project_id

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {"method": "get", "url": "/presigned_url", "params": params}

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[GetPresignedUrlResponse, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = GetPresignedUrlResponse.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[GetPresignedUrlResponse, HTTPValidationError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *, client: AuthenticatedClient, api_url: str, method: Method, bucket_name: str, object_name: str, project_id: str
) -> Response[Union[GetPresignedUrlResponse, HTTPValidationError]]:
    """Get Presigned Url

    Args:
        api_url (str):
        method (Method):
        bucket_name (str):
        object_name (str):
        project_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[GetPresignedUrlResponse, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        api_url=api_url, method=method, bucket_name=bucket_name, object_name=object_name, project_id=project_id
    )

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    *, client: AuthenticatedClient, api_url: str, method: Method, bucket_name: str, object_name: str, project_id: str
) -> Optional[Union[GetPresignedUrlResponse, HTTPValidationError]]:
    """Get Presigned Url

    Args:
        api_url (str):
        method (Method):
        bucket_name (str):
        object_name (str):
        project_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[GetPresignedUrlResponse, HTTPValidationError]
    """

    return sync_detailed(
        client=client,
        api_url=api_url,
        method=method,
        bucket_name=bucket_name,
        object_name=object_name,
        project_id=project_id,
    ).parsed


async def asyncio_detailed(
    *, client: AuthenticatedClient, api_url: str, method: Method, bucket_name: str, object_name: str, project_id: str
) -> Response[Union[GetPresignedUrlResponse, HTTPValidationError]]:
    """Get Presigned Url

    Args:
        api_url (str):
        method (Method):
        bucket_name (str):
        object_name (str):
        project_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[GetPresignedUrlResponse, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        api_url=api_url, method=method, bucket_name=bucket_name, object_name=object_name, project_id=project_id
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *, client: AuthenticatedClient, api_url: str, method: Method, bucket_name: str, object_name: str, project_id: str
) -> Optional[Union[GetPresignedUrlResponse, HTTPValidationError]]:
    """Get Presigned Url

    Args:
        api_url (str):
        method (Method):
        bucket_name (str):
        object_name (str):
        project_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[GetPresignedUrlResponse, HTTPValidationError]
    """

    return (
        await asyncio_detailed(
            client=client,
            api_url=api_url,
            method=method,
            bucket_name=bucket_name,
            object_name=object_name,
            project_id=project_id,
        )
    ).parsed



from typing import Any, Literal, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.user_metadata_filter_operator import UserMetadataFilterOperator
from ..types import UNSET, Unset

T = TypeVar("T", bound="UserMetadataFilter")


@_attrs_define
class UserMetadataFilter:
    """
    Attributes:
        key (str):
        operator (UserMetadataFilterOperator):
        value (Union[list[str], str]):
        name (Union[Literal['user_metadata'], Unset]):  Default: 'user_metadata'.
    """

    key: str
    operator: UserMetadataFilterOperator
    value: Union[list[str], str]
    name: Union[Literal["user_metadata"], Unset] = "user_metadata"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        key = self.key

        operator = self.operator.value

        value: Union[list[str], str]
        if isinstance(self.value, list):
            value = self.value

        else:
            value = self.value

        name = self.name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"key": key, "operator": operator, "value": value})
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        key = d.pop("key")

        operator = UserMetadataFilterOperator(d.pop("operator"))

        def _parse_value(data: object) -> Union[list[str], str]:
            try:
                if not isinstance(data, list):
                    raise TypeError()
                value_type_1 = cast(list[str], data)

                return value_type_1
            except:  # noqa: E722
                pass
            return cast(Union[list[str], str], data)

        value = _parse_value(d.pop("value"))

        name = cast(Union[Literal["user_metadata"], Unset], d.pop("name", UNSET))
        if name != "user_metadata" and not isinstance(name, Unset):
            raise ValueError(f"name must match const 'user_metadata', got '{name}'")

        user_metadata_filter = cls(key=key, operator=operator, value=value, name=name)

        user_metadata_filter.additional_properties = d
        return user_metadata_filter

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.alert_monitor_type import AlertMonitorType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.alert_condition import AlertCondition


T = TypeVar("T", bound="AlertConfigurationResponse")


@_attrs_define
class AlertConfigurationResponse:
    """
    Attributes:
        created_at (datetime.datetime):
        created_by (str):
        id (str):
        project_id (str):
        updated_at (datetime.datetime):
        active (Union[None, Unset, bool]):
        alert_sent (Union[None, Unset, bool]):
        conditions (Union[None, Unset, list['AlertCondition']]):
        counter (Union[None, Unset, int]):
        counter_since_last_sent (Union[None, Unset, int]):
        interval (Union[None, Unset, int]):
        last_sent (Union[None, Unset, datetime.datetime]):
        last_triggered (Union[None, Unset, datetime.datetime]):
        processed_by (Union[None, Unset, str]):
        schema_version (Union[AlertMonitorType, None, Unset]):
    """

    created_at: datetime.datetime
    created_by: str
    id: str
    project_id: str
    updated_at: datetime.datetime
    active: Union[None, Unset, bool] = UNSET
    alert_sent: Union[None, Unset, bool] = UNSET
    conditions: Union[None, Unset, list["AlertCondition"]] = UNSET
    counter: Union[None, Unset, int] = UNSET
    counter_since_last_sent: Union[None, Unset, int] = UNSET
    interval: Union[None, Unset, int] = UNSET
    last_sent: Union[None, Unset, datetime.datetime] = UNSET
    last_triggered: Union[None, Unset, datetime.datetime] = UNSET
    processed_by: Union[None, Unset, str] = UNSET
    schema_version: Union[AlertMonitorType, None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        created_at = self.created_at.isoformat()

        created_by = self.created_by

        id = self.id

        project_id = self.project_id

        updated_at = self.updated_at.isoformat()

        active: Union[None, Unset, bool]
        if isinstance(self.active, Unset):
            active = UNSET
        else:
            active = self.active

        alert_sent: Union[None, Unset, bool]
        if isinstance(self.alert_sent, Unset):
            alert_sent = UNSET
        else:
            alert_sent = self.alert_sent

        conditions: Union[None, Unset, list[dict[str, Any]]]
        if isinstance(self.conditions, Unset):
            conditions = UNSET
        elif isinstance(self.conditions, list):
            conditions = []
            for conditions_type_0_item_data in self.conditions:
                conditions_type_0_item = conditions_type_0_item_data.to_dict()
                conditions.append(conditions_type_0_item)

        else:
            conditions = self.conditions

        counter: Union[None, Unset, int]
        if isinstance(self.counter, Unset):
            counter = UNSET
        else:
            counter = self.counter

        counter_since_last_sent: Union[None, Unset, int]
        if isinstance(self.counter_since_last_sent, Unset):
            counter_since_last_sent = UNSET
        else:
            counter_since_last_sent = self.counter_since_last_sent

        interval: Union[None, Unset, int]
        if isinstance(self.interval, Unset):
            interval = UNSET
        else:
            interval = self.interval

        last_sent: Union[None, Unset, str]
        if isinstance(self.last_sent, Unset):
            last_sent = UNSET
        elif isinstance(self.last_sent, datetime.datetime):
            last_sent = self.last_sent.isoformat()
        else:
            last_sent = self.last_sent

        last_triggered: Union[None, Unset, str]
        if isinstance(self.last_triggered, Unset):
            last_triggered = UNSET
        elif isinstance(self.last_triggered, datetime.datetime):
            last_triggered = self.last_triggered.isoformat()
        else:
            last_triggered = self.last_triggered

        processed_by: Union[None, Unset, str]
        if isinstance(self.processed_by, Unset):
            processed_by = UNSET
        else:
            processed_by = self.processed_by

        schema_version: Union[None, Unset, str]
        if isinstance(self.schema_version, Unset):
            schema_version = UNSET
        elif isinstance(self.schema_version, AlertMonitorType):
            schema_version = self.schema_version.value
        else:
            schema_version = self.schema_version

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "created_at": created_at,
                "created_by": created_by,
                "id": id,
                "project_id": project_id,
                "updated_at": updated_at,
            }
        )
        if active is not UNSET:
            field_dict["active"] = active
        if alert_sent is not UNSET:
            field_dict["alert_sent"] = alert_sent
        if conditions is not UNSET:
            field_dict["conditions"] = conditions
        if counter is not UNSET:
            field_dict["counter"] = counter
        if counter_since_last_sent is not UNSET:
            field_dict["counter_since_last_sent"] = counter_since_last_sent
        if interval is not UNSET:
            field_dict["interval"] = interval
        if last_sent is not UNSET:
            field_dict["last_sent"] = last_sent
        if last_triggered is not UNSET:
            field_dict["last_triggered"] = last_triggered
        if processed_by is not UNSET:
            field_dict["processed_by"] = processed_by
        if schema_version is not UNSET:
            field_dict["schema_version"] = schema_version

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.alert_condition import AlertCondition

        d = src_dict.copy()
        created_at = isoparse(d.pop("created_at"))

        created_by = d.pop("created_by")

        id = d.pop("id")

        project_id = d.pop("project_id")

        updated_at = isoparse(d.pop("updated_at"))

        def _parse_active(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        active = _parse_active(d.pop("active", UNSET))

        def _parse_alert_sent(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        alert_sent = _parse_alert_sent(d.pop("alert_sent", UNSET))

        def _parse_conditions(data: object) -> Union[None, Unset, list["AlertCondition"]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                conditions_type_0 = []
                _conditions_type_0 = data
                for conditions_type_0_item_data in _conditions_type_0:
                    conditions_type_0_item = AlertCondition.from_dict(conditions_type_0_item_data)

                    conditions_type_0.append(conditions_type_0_item)

                return conditions_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list["AlertCondition"]], data)

        conditions = _parse_conditions(d.pop("conditions", UNSET))

        def _parse_counter(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        counter = _parse_counter(d.pop("counter", UNSET))

        def _parse_counter_since_last_sent(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        counter_since_last_sent = _parse_counter_since_last_sent(d.pop("counter_since_last_sent", UNSET))

        def _parse_interval(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        interval = _parse_interval(d.pop("interval", UNSET))

        def _parse_last_sent(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_sent_type_0 = isoparse(data)

                return last_sent_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        last_sent = _parse_last_sent(d.pop("last_sent", UNSET))

        def _parse_last_triggered(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_triggered_type_0 = isoparse(data)

                return last_triggered_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        last_triggered = _parse_last_triggered(d.pop("last_triggered", UNSET))

        def _parse_processed_by(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        processed_by = _parse_processed_by(d.pop("processed_by", UNSET))

        def _parse_schema_version(data: object) -> Union[AlertMonitorType, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                schema_version_type_0 = AlertMonitorType(data)

                return schema_version_type_0
            except:  # noqa: E722
                pass
            return cast(Union[AlertMonitorType, None, Unset], data)

        schema_version = _parse_schema_version(d.pop("schema_version", UNSET))

        alert_configuration_response = cls(
            created_at=created_at,
            created_by=created_by,
            id=id,
            project_id=project_id,
            updated_at=updated_at,
            active=active,
            alert_sent=alert_sent,
            conditions=conditions,
            counter=counter,
            counter_since_last_sent=counter_since_last_sent,
            interval=interval,
            last_sent=last_sent,
            last_triggered=last_triggered,
            processed_by=processed_by,
            schema_version=schema_version,
        )

        alert_configuration_response.additional_properties = d
        return alert_configuration_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.api_key_action import ApiKeyAction
from ..models.dataset_action import DatasetAction
from ..models.generated_scorer_action import GeneratedScorerAction
from ..models.group_action import GroupAction
from ..models.group_member_action import GroupMemberAction
from ..models.organization_action import OrganizationAction
from ..models.project_action import ProjectAction
from ..models.registered_scorer_action import RegisteredScorerAction
from ..models.user_action import UserAction
from ..types import UNSET, Unset

T = TypeVar("T", bound="Permission")


@_attrs_define
class Permission:
    """
    Attributes:
        action (Union[ApiKeyAction, DatasetAction, GeneratedScorerAction, GroupAction, GroupMemberAction,
            OrganizationAction, ProjectAction, RegisteredScorerAction, UserAction]):
        allowed (bool):
        message (Union[None, Unset, str]):
    """

    action: Union[
        ApiKeyAction,
        DatasetAction,
        GeneratedScorerAction,
        GroupAction,
        GroupMemberAction,
        OrganizationAction,
        ProjectAction,
        RegisteredScorerAction,
        UserAction,
    ]
    allowed: bool
    message: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        action: str
        if isinstance(self.action, UserAction):
            action = self.action.value
        elif isinstance(self.action, GroupAction):
            action = self.action.value
        elif isinstance(self.action, GroupMemberAction):
            action = self.action.value
        elif isinstance(self.action, ProjectAction):
            action = self.action.value
        elif isinstance(self.action, RegisteredScorerAction):
            action = self.action.value
        elif isinstance(self.action, ApiKeyAction):
            action = self.action.value
        elif isinstance(self.action, GeneratedScorerAction):
            action = self.action.value
        elif isinstance(self.action, DatasetAction):
            action = self.action.value
        else:
            action = self.action.value

        allowed = self.allowed

        message: Union[None, Unset, str]
        if isinstance(self.message, Unset):
            message = UNSET
        else:
            message = self.message

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"action": action, "allowed": allowed})
        if message is not UNSET:
            field_dict["message"] = message

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_action(
            data: object,
        ) -> Union[
            ApiKeyAction,
            DatasetAction,
            GeneratedScorerAction,
            GroupAction,
            GroupMemberAction,
            OrganizationAction,
            ProjectAction,
            RegisteredScorerAction,
            UserAction,
        ]:
            try:
                if not isinstance(data, str):
                    raise TypeError()
                action_type_0 = UserAction(data)

                return action_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, str):
                    raise TypeError()
                action_type_1 = GroupAction(data)

                return action_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, str):
                    raise TypeError()
                action_type_2 = GroupMemberAction(data)

                return action_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, str):
                    raise TypeError()
                action_type_3 = ProjectAction(data)

                return action_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, str):
                    raise TypeError()
                action_type_4 = RegisteredScorerAction(data)

                return action_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, str):
                    raise TypeError()
                action_type_5 = ApiKeyAction(data)

                return action_type_5
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, str):
                    raise TypeError()
                action_type_6 = GeneratedScorerAction(data)

                return action_type_6
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, str):
                    raise TypeError()
                action_type_7 = DatasetAction(data)

                return action_type_7
            except:  # noqa: E722
                pass
            if not isinstance(data, str):
                raise TypeError()
            action_type_8 = OrganizationAction(data)

            return action_type_8

        action = _parse_action(d.pop("action"))

        allowed = d.pop("allowed")

        def _parse_message(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        message = _parse_message(d.pop("message", UNSET))

        permission = cls(action=action, allowed=allowed, message=message)

        permission.additional_properties = d
        return permission

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="CommunityResponse")


@_attrs_define
class CommunityResponse:
    """See `rungalileo.schemas.content.insights.Community` for details on Communities.

    Before returning the results to the UI, we scale the continuous community score to
    an int for better interpretability for the user.

    0-0.03 = 1
    0.03-0.07 = 2
    0.07-0.15 = 3
    0.15-0.2=4
    0.2=5

        Attributes:
            labels (list[str]):
            num_samples (int):
            score (int):
    """

    labels: list[str]
    num_samples: int
    score: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        labels = self.labels

        num_samples = self.num_samples

        score = self.score

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"labels": labels, "num_samples": num_samples, "score": score})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        labels = cast(list[str], d.pop("labels"))

        num_samples = d.pop("num_samples")

        score = d.pop("score")

        community_response = cls(labels=labels, num_samples=num_samples, score=score)

        community_response.additional_properties = d
        return community_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.metric_critique_content import MetricCritiqueContent


T = TypeVar("T", bound="MetricCritiqueColumnar")


@_attrs_define
class MetricCritiqueColumnar:
    """
    Attributes:
        critique_info (MetricCritiqueContent):
        id (str):
        is_computed (bool):
        revised_explanation (Union[None, str]):
    """

    critique_info: "MetricCritiqueContent"
    id: str
    is_computed: bool
    revised_explanation: Union[None, str]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        critique_info = self.critique_info.to_dict()

        id = self.id

        is_computed = self.is_computed

        revised_explanation: Union[None, str]
        revised_explanation = self.revised_explanation

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "critique_info": critique_info,
                "id": id,
                "is_computed": is_computed,
                "revised_explanation": revised_explanation,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.metric_critique_content import MetricCritiqueContent

        d = src_dict.copy()
        critique_info = MetricCritiqueContent.from_dict(d.pop("critique_info"))

        id = d.pop("id")

        is_computed = d.pop("is_computed")

        def _parse_revised_explanation(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        revised_explanation = _parse_revised_explanation(d.pop("revised_explanation"))

        metric_critique_columnar = cls(
            critique_info=critique_info, id=id, is_computed=is_computed, revised_explanation=revised_explanation
        )

        metric_critique_columnar.additional_properties = d
        return metric_critique_columnar

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.metric_computing import MetricComputing
    from ..models.metric_error import MetricError
    from ..models.metric_failed import MetricFailed
    from ..models.metric_not_applicable import MetricNotApplicable
    from ..models.metric_not_computed import MetricNotComputed
    from ..models.metric_pending import MetricPending
    from ..models.metric_success import MetricSuccess


T = TypeVar("T", bound="LogDataQueryResponse")


@_attrs_define
class LogDataQueryResponse:
    """
    Attributes:
        column_ids (Union[Unset, list[str]]): column ids present in each row
        limit (Union[Unset, int]):  Default: 100.
        next_starting_token (Union[None, Unset, int]):
        num_rows (Union[Unset, int]): number of data rows
        paginated (Union[Unset, bool]):  Default: False.
        rows (Union[Unset, list[list[Union['MetricComputing', 'MetricError', 'MetricFailed', 'MetricNotApplicable',
            'MetricNotComputed', 'MetricPending', 'MetricSuccess']]]]): list of data rows
        starting_token (Union[Unset, int]):  Default: 0.
    """

    column_ids: Union[Unset, list[str]] = UNSET
    limit: Union[Unset, int] = 100
    next_starting_token: Union[None, Unset, int] = UNSET
    num_rows: Union[Unset, int] = UNSET
    paginated: Union[Unset, bool] = False
    rows: Union[
        Unset,
        list[
            list[
                Union[
                    "MetricComputing",
                    "MetricError",
                    "MetricFailed",
                    "MetricNotApplicable",
                    "MetricNotComputed",
                    "MetricPending",
                    "MetricSuccess",
                ]
            ]
        ],
    ] = UNSET
    starting_token: Union[Unset, int] = 0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.metric_computing import MetricComputing
        from ..models.metric_error import MetricError
        from ..models.metric_not_applicable import MetricNotApplicable
        from ..models.metric_not_computed import MetricNotComputed
        from ..models.metric_pending import MetricPending
        from ..models.metric_success import MetricSuccess

        column_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.column_ids, Unset):
            column_ids = self.column_ids

        limit = self.limit

        next_starting_token: Union[None, Unset, int]
        if isinstance(self.next_starting_token, Unset):
            next_starting_token = UNSET
        else:
            next_starting_token = self.next_starting_token

        num_rows = self.num_rows

        paginated = self.paginated

        rows: Union[Unset, list[list[dict[str, Any]]]] = UNSET
        if not isinstance(self.rows, Unset):
            rows = []
            for rows_item_data in self.rows:
                rows_item = []
                for rows_item_item_data in rows_item_data:
                    rows_item_item: dict[str, Any]
                    if isinstance(rows_item_item_data, MetricNotComputed):
                        rows_item_item = rows_item_item_data.to_dict()
                    elif isinstance(rows_item_item_data, MetricPending):
                        rows_item_item = rows_item_item_data.to_dict()
                    elif isinstance(rows_item_item_data, MetricComputing):
                        rows_item_item = rows_item_item_data.to_dict()
                    elif isinstance(rows_item_item_data, MetricNotApplicable):
                        rows_item_item = rows_item_item_data.to_dict()
                    elif isinstance(rows_item_item_data, MetricSuccess):
                        rows_item_item = rows_item_item_data.to_dict()
                    elif isinstance(rows_item_item_data, MetricError):
                        rows_item_item = rows_item_item_data.to_dict()
                    else:
                        rows_item_item = rows_item_item_data.to_dict()

                    rows_item.append(rows_item_item)

                rows.append(rows_item)

        starting_token = self.starting_token

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if column_ids is not UNSET:
            field_dict["column_ids"] = column_ids
        if limit is not UNSET:
            field_dict["limit"] = limit
        if next_starting_token is not UNSET:
            field_dict["next_starting_token"] = next_starting_token
        if num_rows is not UNSET:
            field_dict["num_rows"] = num_rows
        if paginated is not UNSET:
            field_dict["paginated"] = paginated
        if rows is not UNSET:
            field_dict["rows"] = rows
        if starting_token is not UNSET:
            field_dict["starting_token"] = starting_token

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.metric_computing import MetricComputing
        from ..models.metric_error import MetricError
        from ..models.metric_failed import MetricFailed
        from ..models.metric_not_applicable import MetricNotApplicable
        from ..models.metric_not_computed import MetricNotComputed
        from ..models.metric_pending import MetricPending
        from ..models.metric_success import MetricSuccess

        d = src_dict.copy()
        column_ids = cast(list[str], d.pop("column_ids", UNSET))

        limit = d.pop("limit", UNSET)

        def _parse_next_starting_token(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        next_starting_token = _parse_next_starting_token(d.pop("next_starting_token", UNSET))

        num_rows = d.pop("num_rows", UNSET)

        paginated = d.pop("paginated", UNSET)

        rows = []
        _rows = d.pop("rows", UNSET)
        for rows_item_data in _rows or []:
            rows_item = []
            _rows_item = rows_item_data
            for rows_item_item_data in _rows_item:

                def _parse_rows_item_item(
                    data: object,
                ) -> Union[
                    "MetricComputing",
                    "MetricError",
                    "MetricFailed",
                    "MetricNotApplicable",
                    "MetricNotComputed",
                    "MetricPending",
                    "MetricSuccess",
                ]:
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        rows_item_item_type_0 = MetricNotComputed.from_dict(data)

                        return rows_item_item_type_0
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        rows_item_item_type_1 = MetricPending.from_dict(data)

                        return rows_item_item_type_1
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        rows_item_item_type_2 = MetricComputing.from_dict(data)

                        return rows_item_item_type_2
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        rows_item_item_type_3 = MetricNotApplicable.from_dict(data)

                        return rows_item_item_type_3
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        rows_item_item_type_4 = MetricSuccess.from_dict(data)

                        return rows_item_item_type_4
                    except:  # noqa: E722
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        rows_item_item_type_5 = MetricError.from_dict(data)

                        return rows_item_item_type_5
                    except:  # noqa: E722
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    rows_item_item_type_6 = MetricFailed.from_dict(data)

                    return rows_item_item_type_6

                rows_item_item = _parse_rows_item_item(rows_item_item_data)

                rows_item.append(rows_item_item)

            rows.append(rows_item)

        starting_token = d.pop("starting_token", UNSET)

        log_data_query_response = cls(
            column_ids=column_ids,
            limit=limit,
            next_starting_token=next_starting_token,
            num_rows=num_rows,
            paginated=paginated,
            rows=rows,
            starting_token=starting_token,
        )

        log_data_query_response.additional_properties = d
        return log_data_query_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class EditAction(str, Enum):
    ADD_SPAN = "add_span"
    CREATE_NEW_LABEL = "create_new_label"
    DELETE = "delete"
    RELABEL = "relabel"
    RELABEL_AS_PRED = "relabel_as_pred"
    SELECT_FOR_LABEL = "select_for_label"
    SHIFT_SPAN = "shift_span"
    UPDATE_TEXT = "update_text"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ErrorDistribution")


@_attrs_define
class ErrorDistribution:
    """
    Attributes:
        counts (Union[Unset, list[int]]):
        labels (Union[Unset, list[str]]):
        percentage (Union[Unset, float]):  Default: 0.0.
    """

    counts: Union[Unset, list[int]] = UNSET
    labels: Union[Unset, list[str]] = UNSET
    percentage: Union[Unset, float] = 0.0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        counts: Union[Unset, list[int]] = UNSET
        if not isinstance(self.counts, Unset):
            counts = self.counts

        labels: Union[Unset, list[str]] = UNSET
        if not isinstance(self.labels, Unset):
            labels = self.labels

        percentage = self.percentage

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if counts is not UNSET:
            field_dict["counts"] = counts
        if labels is not UNSET:
            field_dict["labels"] = labels
        if percentage is not UNSET:
            field_dict["percentage"] = percentage

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        counts = cast(list[int], d.pop("counts", UNSET))

        labels = cast(list[str], d.pop("labels", UNSET))

        percentage = d.pop("percentage", UNSET)

        error_distribution = cls(counts=counts, labels=labels, percentage=percentage)

        error_distribution.additional_properties = d
        return error_distribution

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.field_type import FieldType
from ..models.operator import Operator
from ..types import UNSET, Unset

T = TypeVar("T", bound="QueryFilterV1")


@_attrs_define
class QueryFilterV1:
    """
    Attributes:
        col_name (str):
        operator (Operator):
        value (Any):
        alternate_json_field (Union[None, Unset, str]):
        json_field (Union[None, Unset, str]):
        json_field_type (Union[FieldType, None, Unset]):
        value_is_column (Union[None, Unset, bool]):  Default: False.
    """

    col_name: str
    operator: Operator
    value: Any
    alternate_json_field: Union[None, Unset, str] = UNSET
    json_field: Union[None, Unset, str] = UNSET
    json_field_type: Union[FieldType, None, Unset] = UNSET
    value_is_column: Union[None, Unset, bool] = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        col_name = self.col_name

        operator = self.operator.value

        value = self.value

        alternate_json_field: Union[None, Unset, str]
        if isinstance(self.alternate_json_field, Unset):
            alternate_json_field = UNSET
        else:
            alternate_json_field = self.alternate_json_field

        json_field: Union[None, Unset, str]
        if isinstance(self.json_field, Unset):
            json_field = UNSET
        else:
            json_field = self.json_field

        json_field_type: Union[None, Unset, str]
        if isinstance(self.json_field_type, Unset):
            json_field_type = UNSET
        elif isinstance(self.json_field_type, FieldType):
            json_field_type = self.json_field_type.value
        else:
            json_field_type = self.json_field_type

        value_is_column: Union[None, Unset, bool]
        if isinstance(self.value_is_column, Unset):
            value_is_column = UNSET
        else:
            value_is_column = self.value_is_column

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"col_name": col_name, "operator": operator, "value": value})
        if alternate_json_field is not UNSET:
            field_dict["alternate_json_field"] = alternate_json_field
        if json_field is not UNSET:
            field_dict["json_field"] = json_field
        if json_field_type is not UNSET:
            field_dict["json_field_type"] = json_field_type
        if value_is_column is not UNSET:
            field_dict["value_is_column"] = value_is_column

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        col_name = d.pop("col_name")

        operator = Operator(d.pop("operator"))

        value = d.pop("value")

        def _parse_alternate_json_field(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        alternate_json_field = _parse_alternate_json_field(d.pop("alternate_json_field", UNSET))

        def _parse_json_field(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        json_field = _parse_json_field(d.pop("json_field", UNSET))

        def _parse_json_field_type(data: object) -> Union[FieldType, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                json_field_type_type_0 = FieldType(data)

                return json_field_type_type_0
            except:  # noqa: E722
                pass
            return cast(Union[FieldType, None, Unset], data)

        json_field_type = _parse_json_field_type(d.pop("json_field_type", UNSET))

        def _parse_value_is_column(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        value_is_column = _parse_value_is_column(d.pop("value_is_column", UNSET))

        query_filter_v1 = cls(
            col_name=col_name,
            operator=operator,
            value=value,
            alternate_json_field=alternate_json_field,
            json_field=json_field,
            json_field_type=json_field_type,
            value_is_column=value_is_column,
        )

        query_filter_v1.additional_properties = d
        return query_filter_v1

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.top_k_token import TopKToken


T = TypeVar("T", bound="Seq2SeqSegment")


@_attrs_define
class Seq2SeqSegment:
    """
    Attributes:
        end (int):
        start (int):
        data_error_potential (Union[None, Unset, float]):
        generated_uncertainty (Union[None, Unset, float]):
        segment_probability (Union[None, Unset, float]):
        top_k_tokens (Union[Unset, list['TopKToken']]):
    """

    end: int
    start: int
    data_error_potential: Union[None, Unset, float] = UNSET
    generated_uncertainty: Union[None, Unset, float] = UNSET
    segment_probability: Union[None, Unset, float] = UNSET
    top_k_tokens: Union[Unset, list["TopKToken"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        end = self.end

        start = self.start

        data_error_potential: Union[None, Unset, float]
        if isinstance(self.data_error_potential, Unset):
            data_error_potential = UNSET
        else:
            data_error_potential = self.data_error_potential

        generated_uncertainty: Union[None, Unset, float]
        if isinstance(self.generated_uncertainty, Unset):
            generated_uncertainty = UNSET
        else:
            generated_uncertainty = self.generated_uncertainty

        segment_probability: Union[None, Unset, float]
        if isinstance(self.segment_probability, Unset):
            segment_probability = UNSET
        else:
            segment_probability = self.segment_probability

        top_k_tokens: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.top_k_tokens, Unset):
            top_k_tokens = []
            for top_k_tokens_item_data in self.top_k_tokens:
                top_k_tokens_item = top_k_tokens_item_data.to_dict()
                top_k_tokens.append(top_k_tokens_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"end": end, "start": start})
        if data_error_potential is not UNSET:
            field_dict["data_error_potential"] = data_error_potential
        if generated_uncertainty is not UNSET:
            field_dict["generated_uncertainty"] = generated_uncertainty
        if segment_probability is not UNSET:
            field_dict["segment_probability"] = segment_probability
        if top_k_tokens is not UNSET:
            field_dict["top_k_tokens"] = top_k_tokens

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.top_k_token import TopKToken

        d = src_dict.copy()
        end = d.pop("end")

        start = d.pop("start")

        def _parse_data_error_potential(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        data_error_potential = _parse_data_error_potential(d.pop("data_error_potential", UNSET))

        def _parse_generated_uncertainty(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        generated_uncertainty = _parse_generated_uncertainty(d.pop("generated_uncertainty", UNSET))

        def _parse_segment_probability(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        segment_probability = _parse_segment_probability(d.pop("segment_probability", UNSET))

        top_k_tokens = []
        _top_k_tokens = d.pop("top_k_tokens", UNSET)
        for top_k_tokens_item_data in _top_k_tokens or []:
            top_k_tokens_item = TopKToken.from_dict(top_k_tokens_item_data)

            top_k_tokens.append(top_k_tokens_item)

        seq_2_seq_segment = cls(
            end=end,
            start=start,
            data_error_potential=data_error_potential,
            generated_uncertainty=generated_uncertainty,
            segment_probability=segment_probability,
            top_k_tokens=top_k_tokens,
        )

        seq_2_seq_segment.additional_properties = d
        return seq_2_seq_segment

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.agent_step import AgentStep
    from ..models.chain_step import ChainStep
    from ..models.llm_step import LlmStep
    from ..models.retriever_step import RetrieverStep
    from ..models.tool_step import ToolStep
    from ..models.workflow_step import WorkflowStep


T = TypeVar("T", bound="WorkflowsIngestRequest")


@_attrs_define
class WorkflowsIngestRequest:
    """
    Attributes:
        workflows (list[Union['AgentStep', 'ChainStep', 'LlmStep', 'RetrieverStep', 'ToolStep', 'WorkflowStep']]): List
            of workflows to log.
        project_id (Union[None, Unset, str]): Project ID for the Observe project.
        project_name (Union[None, Unset, str]): Project name for the Observe project.
    """

    workflows: list[Union["AgentStep", "ChainStep", "LlmStep", "RetrieverStep", "ToolStep", "WorkflowStep"]]
    project_id: Union[None, Unset, str] = UNSET
    project_name: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.chain_step import ChainStep
        from ..models.llm_step import LlmStep
        from ..models.retriever_step import RetrieverStep
        from ..models.tool_step import ToolStep
        from ..models.workflow_step import WorkflowStep

        workflows = []
        for workflows_item_data in self.workflows:
            workflows_item: dict[str, Any]
            if isinstance(workflows_item_data, WorkflowStep):
                workflows_item = workflows_item_data.to_dict()
            elif isinstance(workflows_item_data, ChainStep):
                workflows_item = workflows_item_data.to_dict()
            elif isinstance(workflows_item_data, LlmStep):
                workflows_item = workflows_item_data.to_dict()
            elif isinstance(workflows_item_data, RetrieverStep):
                workflows_item = workflows_item_data.to_dict()
            elif isinstance(workflows_item_data, ToolStep):
                workflows_item = workflows_item_data.to_dict()
            else:
                workflows_item = workflows_item_data.to_dict()

            workflows.append(workflows_item)

        project_id: Union[None, Unset, str]
        if isinstance(self.project_id, Unset):
            project_id = UNSET
        else:
            project_id = self.project_id

        project_name: Union[None, Unset, str]
        if isinstance(self.project_name, Unset):
            project_name = UNSET
        else:
            project_name = self.project_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"workflows": workflows})
        if project_id is not UNSET:
            field_dict["project_id"] = project_id
        if project_name is not UNSET:
            field_dict["project_name"] = project_name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.agent_step import AgentStep
        from ..models.chain_step import ChainStep
        from ..models.llm_step import LlmStep
        from ..models.retriever_step import RetrieverStep
        from ..models.tool_step import ToolStep
        from ..models.workflow_step import WorkflowStep

        d = src_dict.copy()
        workflows = []
        _workflows = d.pop("workflows")
        for workflows_item_data in _workflows:

            def _parse_workflows_item(
                data: object,
            ) -> Union["AgentStep", "ChainStep", "LlmStep", "RetrieverStep", "ToolStep", "WorkflowStep"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    workflows_item_type_0 = WorkflowStep.from_dict(data)

                    return workflows_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    workflows_item_type_1 = ChainStep.from_dict(data)

                    return workflows_item_type_1
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    workflows_item_type_2 = LlmStep.from_dict(data)

                    return workflows_item_type_2
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    workflows_item_type_3 = RetrieverStep.from_dict(data)

                    return workflows_item_type_3
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    workflows_item_type_4 = ToolStep.from_dict(data)

                    return workflows_item_type_4
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                workflows_item_type_5 = AgentStep.from_dict(data)

                return workflows_item_type_5

            workflows_item = _parse_workflows_item(workflows_item_data)

            workflows.append(workflows_item)

        def _parse_project_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        project_id = _parse_project_id(d.pop("project_id", UNSET))

        def _parse_project_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        project_name = _parse_project_name(d.pop("project_name", UNSET))

        workflows_ingest_request = cls(workflows=workflows, project_id=project_id, project_name=project_name)

        workflows_ingest_request.additional_properties = d
        return workflows_ingest_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Literal, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.agent_step import AgentStep
    from ..models.chain_step import ChainStep
    from ..models.document import Document
    from ..models.llm_step import LlmStep
    from ..models.message import Message
    from ..models.retriever_step import RetrieverStep
    from ..models.step_with_children import StepWithChildren
    from ..models.tool_step import ToolStep
    from ..models.workflow_step_input_type_3 import WorkflowStepInputType3
    from ..models.workflow_step_input_type_7_item import WorkflowStepInputType7Item
    from ..models.workflow_step_input_type_8_item import WorkflowStepInputType8Item
    from ..models.workflow_step_metadata import WorkflowStepMetadata
    from ..models.workflow_step_output_type_3 import WorkflowStepOutputType3
    from ..models.workflow_step_output_type_7_item import WorkflowStepOutputType7Item
    from ..models.workflow_step_output_type_8_item import WorkflowStepOutputType8Item


T = TypeVar("T", bound="WorkflowStep")


@_attrs_define
class WorkflowStep:
    """
    Attributes:
        input_ (Union['Document', 'Message', 'WorkflowStepInputType3', list['Document'], list['Message'],
            list['WorkflowStepInputType7Item'], list['WorkflowStepInputType8Item'], list[str], str]): Input to the step.
        created_at_ns (Union[Unset, int]): Timestamp of the step's creation, as nanoseconds since epoch.
        duration_ns (Union[Unset, int]): Duration of the step in nanoseconds. Default: 0.
        ground_truth (Union[None, Unset, str]): Ground truth expected output for the step.
        metadata (Union[Unset, WorkflowStepMetadata]): Metadata associated with this step.
        name (Union[Unset, str]): Name of the step. Default: ''.
        output (Union['Document', 'Message', 'WorkflowStepOutputType3', Unset, list['Document'], list['Message'],
            list['WorkflowStepOutputType7Item'], list['WorkflowStepOutputType8Item'], list[str], str]): Output of the step.
            Default: ''.
        parent (Union['StepWithChildren', None, Unset]): Parent node of the current node. For internal use only.
        status_code (Union[None, Unset, int]): Status code of the step. Used for logging failed/errored steps.
        steps (Union[Unset, list[Union['AgentStep', 'ChainStep', 'LlmStep', 'RetrieverStep', 'ToolStep',
            'WorkflowStep']]]): Steps in the workflow.
        type_ (Union[Literal['workflow'], Unset]): Type of the step. By default, it is set to workflow. Default:
            'workflow'.
    """

    input_: Union[
        "Document",
        "Message",
        "WorkflowStepInputType3",
        list["Document"],
        list["Message"],
        list["WorkflowStepInputType7Item"],
        list["WorkflowStepInputType8Item"],
        list[str],
        str,
    ]
    created_at_ns: Union[Unset, int] = UNSET
    duration_ns: Union[Unset, int] = 0
    ground_truth: Union[None, Unset, str] = UNSET
    metadata: Union[Unset, "WorkflowStepMetadata"] = UNSET
    name: Union[Unset, str] = ""
    output: Union[
        "Document",
        "Message",
        "WorkflowStepOutputType3",
        Unset,
        list["Document"],
        list["Message"],
        list["WorkflowStepOutputType7Item"],
        list["WorkflowStepOutputType8Item"],
        list[str],
        str,
    ] = ""
    parent: Union["StepWithChildren", None, Unset] = UNSET
    status_code: Union[None, Unset, int] = UNSET
    steps: Union[
        Unset, list[Union["AgentStep", "ChainStep", "LlmStep", "RetrieverStep", "ToolStep", "WorkflowStep"]]
    ] = UNSET
    type_: Union[Literal["workflow"], Unset] = "workflow"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.chain_step import ChainStep
        from ..models.document import Document
        from ..models.llm_step import LlmStep
        from ..models.message import Message
        from ..models.retriever_step import RetrieverStep
        from ..models.step_with_children import StepWithChildren
        from ..models.tool_step import ToolStep
        from ..models.workflow_step_input_type_3 import WorkflowStepInputType3
        from ..models.workflow_step_output_type_3 import WorkflowStepOutputType3

        input_: Union[dict[str, Any], list[dict[str, Any]], list[str], str]
        if isinstance(self.input_, Document):
            input_ = self.input_.to_dict()
        elif isinstance(self.input_, Message):
            input_ = self.input_.to_dict()
        elif isinstance(self.input_, WorkflowStepInputType3):
            input_ = self.input_.to_dict()
        elif isinstance(self.input_, list):
            input_ = self.input_

        elif isinstance(self.input_, list):
            input_ = []
            for input_type_5_item_data in self.input_:
                input_type_5_item = input_type_5_item_data.to_dict()
                input_.append(input_type_5_item)

        elif isinstance(self.input_, list):
            input_ = []
            for input_type_6_item_data in self.input_:
                input_type_6_item = input_type_6_item_data.to_dict()
                input_.append(input_type_6_item)

        elif isinstance(self.input_, list):
            input_ = []
            for input_type_7_item_data in self.input_:
                input_type_7_item = input_type_7_item_data.to_dict()
                input_.append(input_type_7_item)

        elif isinstance(self.input_, list):
            input_ = []
            for input_type_8_item_data in self.input_:
                input_type_8_item = input_type_8_item_data.to_dict()
                input_.append(input_type_8_item)

        else:
            input_ = self.input_

        created_at_ns = self.created_at_ns

        duration_ns = self.duration_ns

        ground_truth: Union[None, Unset, str]
        if isinstance(self.ground_truth, Unset):
            ground_truth = UNSET
        else:
            ground_truth = self.ground_truth

        metadata: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        name = self.name

        output: Union[Unset, dict[str, Any], list[dict[str, Any]], list[str], str]
        if isinstance(self.output, Unset):
            output = UNSET
        elif isinstance(self.output, Document):
            output = self.output.to_dict()
        elif isinstance(self.output, Message):
            output = self.output.to_dict()
        elif isinstance(self.output, WorkflowStepOutputType3):
            output = self.output.to_dict()
        elif isinstance(self.output, list):
            output = self.output

        elif isinstance(self.output, list):
            output = []
            for output_type_5_item_data in self.output:
                output_type_5_item = output_type_5_item_data.to_dict()
                output.append(output_type_5_item)

        elif isinstance(self.output, list):
            output = []
            for output_type_6_item_data in self.output:
                output_type_6_item = output_type_6_item_data.to_dict()
                output.append(output_type_6_item)

        elif isinstance(self.output, list):
            output = []
            for output_type_7_item_data in self.output:
                output_type_7_item = output_type_7_item_data.to_dict()
                output.append(output_type_7_item)

        elif isinstance(self.output, list):
            output = []
            for output_type_8_item_data in self.output:
                output_type_8_item = output_type_8_item_data.to_dict()
                output.append(output_type_8_item)

        else:
            output = self.output

        parent: Union[None, Unset, dict[str, Any]]
        if isinstance(self.parent, Unset):
            parent = UNSET
        elif isinstance(self.parent, StepWithChildren):
            parent = self.parent.to_dict()
        else:
            parent = self.parent

        status_code: Union[None, Unset, int]
        if isinstance(self.status_code, Unset):
            status_code = UNSET
        else:
            status_code = self.status_code

        steps: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.steps, Unset):
            steps = []
            for steps_item_data in self.steps:
                steps_item: dict[str, Any]
                if isinstance(steps_item_data, WorkflowStep):
                    steps_item = steps_item_data.to_dict()
                elif isinstance(steps_item_data, ChainStep):
                    steps_item = steps_item_data.to_dict()
                elif isinstance(steps_item_data, LlmStep):
                    steps_item = steps_item_data.to_dict()
                elif isinstance(steps_item_data, RetrieverStep):
                    steps_item = steps_item_data.to_dict()
                elif isinstance(steps_item_data, ToolStep):
                    steps_item = steps_item_data.to_dict()
                else:
                    steps_item = steps_item_data.to_dict()

                steps.append(steps_item)

        type_ = self.type_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"input": input_})
        if created_at_ns is not UNSET:
            field_dict["created_at_ns"] = created_at_ns
        if duration_ns is not UNSET:
            field_dict["duration_ns"] = duration_ns
        if ground_truth is not UNSET:
            field_dict["ground_truth"] = ground_truth
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if name is not UNSET:
            field_dict["name"] = name
        if output is not UNSET:
            field_dict["output"] = output
        if parent is not UNSET:
            field_dict["parent"] = parent
        if status_code is not UNSET:
            field_dict["status_code"] = status_code
        if steps is not UNSET:
            field_dict["steps"] = steps
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.agent_step import AgentStep
        from ..models.chain_step import ChainStep
        from ..models.document import Document
        from ..models.llm_step import LlmStep
        from ..models.message import Message
        from ..models.retriever_step import RetrieverStep
        from ..models.step_with_children import StepWithChildren
        from ..models.tool_step import ToolStep
        from ..models.workflow_step_input_type_3 import WorkflowStepInputType3
        from ..models.workflow_step_input_type_7_item import WorkflowStepInputType7Item
        from ..models.workflow_step_input_type_8_item import WorkflowStepInputType8Item
        from ..models.workflow_step_metadata import WorkflowStepMetadata
        from ..models.workflow_step_output_type_3 import WorkflowStepOutputType3
        from ..models.workflow_step_output_type_7_item import WorkflowStepOutputType7Item
        from ..models.workflow_step_output_type_8_item import WorkflowStepOutputType8Item

        d = src_dict.copy()

        def _parse_input_(
            data: object,
        ) -> Union[
            "Document",
            "Message",
            "WorkflowStepInputType3",
            list["Document"],
            list["Message"],
            list["WorkflowStepInputType7Item"],
            list["WorkflowStepInputType8Item"],
            list[str],
            str,
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                input_type_1 = Document.from_dict(data)

                return input_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                input_type_2 = Message.from_dict(data)

                return input_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                input_type_3 = WorkflowStepInputType3.from_dict(data)

                return input_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                input_type_4 = cast(list[str], data)

                return input_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                input_type_5 = []
                _input_type_5 = data
                for input_type_5_item_data in _input_type_5:
                    input_type_5_item = Document.from_dict(input_type_5_item_data)

                    input_type_5.append(input_type_5_item)

                return input_type_5
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                input_type_6 = []
                _input_type_6 = data
                for input_type_6_item_data in _input_type_6:
                    input_type_6_item = Message.from_dict(input_type_6_item_data)

                    input_type_6.append(input_type_6_item)

                return input_type_6
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                input_type_7 = []
                _input_type_7 = data
                for input_type_7_item_data in _input_type_7:
                    input_type_7_item = WorkflowStepInputType7Item.from_dict(input_type_7_item_data)

                    input_type_7.append(input_type_7_item)

                return input_type_7
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                input_type_8 = []
                _input_type_8 = data
                for input_type_8_item_data in _input_type_8:
                    input_type_8_item = WorkflowStepInputType8Item.from_dict(input_type_8_item_data)

                    input_type_8.append(input_type_8_item)

                return input_type_8
            except:  # noqa: E722
                pass
            return cast(
                Union[
                    "Document",
                    "Message",
                    "WorkflowStepInputType3",
                    list["Document"],
                    list["Message"],
                    list["WorkflowStepInputType7Item"],
                    list["WorkflowStepInputType8Item"],
                    list[str],
                    str,
                ],
                data,
            )

        input_ = _parse_input_(d.pop("input"))

        created_at_ns = d.pop("created_at_ns", UNSET)

        duration_ns = d.pop("duration_ns", UNSET)

        def _parse_ground_truth(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        ground_truth = _parse_ground_truth(d.pop("ground_truth", UNSET))

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, WorkflowStepMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = WorkflowStepMetadata.from_dict(_metadata)

        name = d.pop("name", UNSET)

        def _parse_output(
            data: object,
        ) -> Union[
            "Document",
            "Message",
            "WorkflowStepOutputType3",
            Unset,
            list["Document"],
            list["Message"],
            list["WorkflowStepOutputType7Item"],
            list["WorkflowStepOutputType8Item"],
            list[str],
            str,
        ]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                output_type_1 = Document.from_dict(data)

                return output_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                output_type_2 = Message.from_dict(data)

                return output_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                output_type_3 = WorkflowStepOutputType3.from_dict(data)

                return output_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                output_type_4 = cast(list[str], data)

                return output_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                output_type_5 = []
                _output_type_5 = data
                for output_type_5_item_data in _output_type_5:
                    output_type_5_item = Document.from_dict(output_type_5_item_data)

                    output_type_5.append(output_type_5_item)

                return output_type_5
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                output_type_6 = []
                _output_type_6 = data
                for output_type_6_item_data in _output_type_6:
                    output_type_6_item = Message.from_dict(output_type_6_item_data)

                    output_type_6.append(output_type_6_item)

                return output_type_6
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                output_type_7 = []
                _output_type_7 = data
                for output_type_7_item_data in _output_type_7:
                    output_type_7_item = WorkflowStepOutputType7Item.from_dict(output_type_7_item_data)

                    output_type_7.append(output_type_7_item)

                return output_type_7
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                output_type_8 = []
                _output_type_8 = data
                for output_type_8_item_data in _output_type_8:
                    output_type_8_item = WorkflowStepOutputType8Item.from_dict(output_type_8_item_data)

                    output_type_8.append(output_type_8_item)

                return output_type_8
            except:  # noqa: E722
                pass
            return cast(
                Union[
                    "Document",
                    "Message",
                    "WorkflowStepOutputType3",
                    Unset,
                    list["Document"],
                    list["Message"],
                    list["WorkflowStepOutputType7Item"],
                    list["WorkflowStepOutputType8Item"],
                    list[str],
                    str,
                ],
                data,
            )

        output = _parse_output(d.pop("output", UNSET))

        def _parse_parent(data: object) -> Union["StepWithChildren", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                parent_type_0 = StepWithChildren.from_dict(data)

                return parent_type_0
            except:  # noqa: E722
                pass
            return cast(Union["StepWithChildren", None, Unset], data)

        parent = _parse_parent(d.pop("parent", UNSET))

        def _parse_status_code(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        status_code = _parse_status_code(d.pop("status_code", UNSET))

        steps = []
        _steps = d.pop("steps", UNSET)
        for steps_item_data in _steps or []:

            def _parse_steps_item(
                data: object,
            ) -> Union["AgentStep", "ChainStep", "LlmStep", "RetrieverStep", "ToolStep", "WorkflowStep"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    steps_item_type_0 = WorkflowStep.from_dict(data)

                    return steps_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    steps_item_type_1 = ChainStep.from_dict(data)

                    return steps_item_type_1
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    steps_item_type_2 = LlmStep.from_dict(data)

                    return steps_item_type_2
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    steps_item_type_3 = RetrieverStep.from_dict(data)

                    return steps_item_type_3
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    steps_item_type_4 = ToolStep.from_dict(data)

                    return steps_item_type_4
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                steps_item_type_5 = AgentStep.from_dict(data)

                return steps_item_type_5

            steps_item = _parse_steps_item(steps_item_data)

            steps.append(steps_item)

        type_ = cast(Union[Literal["workflow"], Unset], d.pop("type", UNSET))
        if type_ != "workflow" and not isinstance(type_, Unset):
            raise ValueError(f"type must match const 'workflow', got '{type_}'")

        workflow_step = cls(
            input_=input_,
            created_at_ns=created_at_ns,
            duration_ns=duration_ns,
            ground_truth=ground_truth,
            metadata=metadata,
            name=name,
            output=output,
            parent=parent,
            status_code=status_code,
            steps=steps,
            type_=type_,
        )

        workflow_step.additional_properties = d
        return workflow_step

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class Split(str, Enum):
    INFERENCE = "inference"
    TEST = "test"
    TRAINING = "training"
    VALIDATION = "validation"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.categorical_prompt_filter_param import CategoricalPromptFilterParam
    from ..models.range_prompt_filter_param import RangePromptFilterParam
    from ..models.value_prompt_filter_param import ValuePromptFilterParam


T = TypeVar("T", bound="PromptFilterParams")


@_attrs_define
class PromptFilterParams:
    """
    Attributes:
        filters (Union[Unset, list[Union['CategoricalPromptFilterParam', 'RangePromptFilterParam',
            'ValuePromptFilterParam']]]): A list of filters.
    """

    filters: Union[
        Unset, list[Union["CategoricalPromptFilterParam", "RangePromptFilterParam", "ValuePromptFilterParam"]]
    ] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.range_prompt_filter_param import RangePromptFilterParam
        from ..models.value_prompt_filter_param import ValuePromptFilterParam

        filters: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = []
            for filters_item_data in self.filters:
                filters_item: dict[str, Any]
                if isinstance(filters_item_data, RangePromptFilterParam):
                    filters_item = filters_item_data.to_dict()
                elif isinstance(filters_item_data, ValuePromptFilterParam):
                    filters_item = filters_item_data.to_dict()
                else:
                    filters_item = filters_item_data.to_dict()

                filters.append(filters_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if filters is not UNSET:
            field_dict["filters"] = filters

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.categorical_prompt_filter_param import CategoricalPromptFilterParam
        from ..models.range_prompt_filter_param import RangePromptFilterParam
        from ..models.value_prompt_filter_param import ValuePromptFilterParam

        d = src_dict.copy()
        filters = []
        _filters = d.pop("filters", UNSET)
        for filters_item_data in _filters or []:

            def _parse_filters_item(
                data: object,
            ) -> Union["CategoricalPromptFilterParam", "RangePromptFilterParam", "ValuePromptFilterParam"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    filters_item_type_0 = RangePromptFilterParam.from_dict(data)

                    return filters_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    filters_item_type_1 = ValuePromptFilterParam.from_dict(data)

                    return filters_item_type_1
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                filters_item_type_2 = CategoricalPromptFilterParam.from_dict(data)

                return filters_item_type_2

            filters_item = _parse_filters_item(filters_item_data)

            filters.append(filters_item)

        prompt_filter_params = cls(filters=filters)

        prompt_filter_params.additional_properties = d
        return prompt_filter_params

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.evaluate_project_settings_metric_weights_configuration_type_0 import (
        EvaluateProjectSettingsMetricWeightsConfigurationType0,
    )
    from ..models.scorers_config import ScorersConfig


T = TypeVar("T", bound="EvaluateProjectSettings")


@_attrs_define
class EvaluateProjectSettings:
    """
    Attributes:
        metric_weights_configuration (Union['EvaluateProjectSettingsMetricWeightsConfigurationType0', None, Unset]):
        scorers_config (Union['ScorersConfig', None, Unset]):
    """

    metric_weights_configuration: Union["EvaluateProjectSettingsMetricWeightsConfigurationType0", None, Unset] = UNSET
    scorers_config: Union["ScorersConfig", None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.evaluate_project_settings_metric_weights_configuration_type_0 import (
            EvaluateProjectSettingsMetricWeightsConfigurationType0,
        )
        from ..models.scorers_config import ScorersConfig

        metric_weights_configuration: Union[None, Unset, dict[str, Any]]
        if isinstance(self.metric_weights_configuration, Unset):
            metric_weights_configuration = UNSET
        elif isinstance(self.metric_weights_configuration, EvaluateProjectSettingsMetricWeightsConfigurationType0):
            metric_weights_configuration = self.metric_weights_configuration.to_dict()
        else:
            metric_weights_configuration = self.metric_weights_configuration

        scorers_config: Union[None, Unset, dict[str, Any]]
        if isinstance(self.scorers_config, Unset):
            scorers_config = UNSET
        elif isinstance(self.scorers_config, ScorersConfig):
            scorers_config = self.scorers_config.to_dict()
        else:
            scorers_config = self.scorers_config

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if metric_weights_configuration is not UNSET:
            field_dict["metric_weights_configuration"] = metric_weights_configuration
        if scorers_config is not UNSET:
            field_dict["scorers_config"] = scorers_config

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.evaluate_project_settings_metric_weights_configuration_type_0 import (
            EvaluateProjectSettingsMetricWeightsConfigurationType0,
        )
        from ..models.scorers_config import ScorersConfig

        d = src_dict.copy()

        def _parse_metric_weights_configuration(
            data: object,
        ) -> Union["EvaluateProjectSettingsMetricWeightsConfigurationType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metric_weights_configuration_type_0 = EvaluateProjectSettingsMetricWeightsConfigurationType0.from_dict(
                    data
                )

                return metric_weights_configuration_type_0
            except:  # noqa: E722
                pass
            return cast(Union["EvaluateProjectSettingsMetricWeightsConfigurationType0", None, Unset], data)

        metric_weights_configuration = _parse_metric_weights_configuration(d.pop("metric_weights_configuration", UNSET))

        def _parse_scorers_config(data: object) -> Union["ScorersConfig", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                scorers_config_type_0 = ScorersConfig.from_dict(data)

                return scorers_config_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ScorersConfig", None, Unset], data)

        scorers_config = _parse_scorers_config(d.pop("scorers_config", UNSET))

        evaluate_project_settings = cls(
            metric_weights_configuration=metric_weights_configuration, scorers_config=scorers_config
        )

        evaluate_project_settings.additional_properties = d
        return evaluate_project_settings

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Literal, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.document import Document
    from ..models.retriever_step_metadata import RetrieverStepMetadata


T = TypeVar("T", bound="RetrieverStep")


@_attrs_define
class RetrieverStep:
    """
    Attributes:
        input_ (str): Input query to the retriever.
        created_at_ns (Union[Unset, int]): Timestamp of the step's creation, as nanoseconds since epoch.
        duration_ns (Union[Unset, int]): Duration of the step in nanoseconds. Default: 0.
        ground_truth (Union[None, Unset, str]): Ground truth expected output for the step.
        metadata (Union[Unset, RetrieverStepMetadata]): Metadata associated with this step.
        name (Union[Unset, str]): Name of the step. Default: ''.
        output (Union[Unset, list['Document']]): Documents retrieved from the retriever. This can be a list of strings
            or `Document`s.
        status_code (Union[None, Unset, int]): Status code of the step. Used for logging failed/errored steps.
        type_ (Union[Literal['retriever'], Unset]): Type of the step. By default, it is set to retriever. Default:
            'retriever'.
    """

    input_: str
    created_at_ns: Union[Unset, int] = UNSET
    duration_ns: Union[Unset, int] = 0
    ground_truth: Union[None, Unset, str] = UNSET
    metadata: Union[Unset, "RetrieverStepMetadata"] = UNSET
    name: Union[Unset, str] = ""
    output: Union[Unset, list["Document"]] = UNSET
    status_code: Union[None, Unset, int] = UNSET
    type_: Union[Literal["retriever"], Unset] = "retriever"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        input_ = self.input_

        created_at_ns = self.created_at_ns

        duration_ns = self.duration_ns

        ground_truth: Union[None, Unset, str]
        if isinstance(self.ground_truth, Unset):
            ground_truth = UNSET
        else:
            ground_truth = self.ground_truth

        metadata: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        name = self.name

        output: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.output, Unset):
            output = []
            for output_item_data in self.output:
                output_item = output_item_data.to_dict()
                output.append(output_item)

        status_code: Union[None, Unset, int]
        if isinstance(self.status_code, Unset):
            status_code = UNSET
        else:
            status_code = self.status_code

        type_ = self.type_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"input": input_})
        if created_at_ns is not UNSET:
            field_dict["created_at_ns"] = created_at_ns
        if duration_ns is not UNSET:
            field_dict["duration_ns"] = duration_ns
        if ground_truth is not UNSET:
            field_dict["ground_truth"] = ground_truth
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if name is not UNSET:
            field_dict["name"] = name
        if output is not UNSET:
            field_dict["output"] = output
        if status_code is not UNSET:
            field_dict["status_code"] = status_code
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.document import Document
        from ..models.retriever_step_metadata import RetrieverStepMetadata

        d = src_dict.copy()
        input_ = d.pop("input")

        created_at_ns = d.pop("created_at_ns", UNSET)

        duration_ns = d.pop("duration_ns", UNSET)

        def _parse_ground_truth(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        ground_truth = _parse_ground_truth(d.pop("ground_truth", UNSET))

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, RetrieverStepMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = RetrieverStepMetadata.from_dict(_metadata)

        name = d.pop("name", UNSET)

        output = []
        _output = d.pop("output", UNSET)
        for output_item_data in _output or []:
            output_item = Document.from_dict(output_item_data)

            output.append(output_item)

        def _parse_status_code(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        status_code = _parse_status_code(d.pop("status_code", UNSET))

        type_ = cast(Union[Literal["retriever"], Unset], d.pop("type", UNSET))
        if type_ != "retriever" and not isinstance(type_, Unset):
            raise ValueError(f"type must match const 'retriever', got '{type_}'")

        retriever_step = cls(
            input_=input_,
            created_at_ns=created_at_ns,
            duration_ns=duration_ns,
            ground_truth=ground_truth,
            metadata=metadata,
            name=name,
            output=output,
            status_code=status_code,
            type_=type_,
        )

        retriever_step.additional_properties = d
        return retriever_step

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="AggregateMetrics")


@_attrs_define
class AggregateMetrics:
    """
    Attributes:
        average_agentic_workflow_success (Union[None, Unset, float]):
        average_completeness_gpt (Union[None, Unset, float]):
        average_factuality (Union[None, Unset, float]):
        average_groundedness (Union[None, Unset, float]):
        average_input_sexist (Union[None, Unset, float]):
        average_input_token_length (Union[None, Unset, float]):
        average_input_toxicity (Union[None, Unset, float]):
        average_instruction_adherence (Union[None, Unset, float]):
        average_latency (Union[None, Unset, float]):
        average_output_token_length (Union[None, Unset, float]):
        average_prompt_perplexity (Union[None, Unset, float]):
        average_retriever_attribution (Union[None, Unset, float]):
        average_retriever_utilization (Union[None, Unset, float]):
        average_sexist (Union[None, Unset, float]):
        average_tool_error_rate (Union[None, Unset, float]):
        average_tool_selection_quality (Union[None, Unset, float]):
        average_total_token_length (Union[None, Unset, float]):
        average_toxicity (Union[None, Unset, float]):
        failures_count (Union[None, Unset, int]):  Default: 0.
        requests_count (Union[None, Unset, int]):  Default: 0.
        total_cost (Union[None, Unset, float]):  Default: 0.0.
    """

    average_agentic_workflow_success: Union[None, Unset, float] = UNSET
    average_completeness_gpt: Union[None, Unset, float] = UNSET
    average_factuality: Union[None, Unset, float] = UNSET
    average_groundedness: Union[None, Unset, float] = UNSET
    average_input_sexist: Union[None, Unset, float] = UNSET
    average_input_token_length: Union[None, Unset, float] = UNSET
    average_input_toxicity: Union[None, Unset, float] = UNSET
    average_instruction_adherence: Union[None, Unset, float] = UNSET
    average_latency: Union[None, Unset, float] = UNSET
    average_output_token_length: Union[None, Unset, float] = UNSET
    average_prompt_perplexity: Union[None, Unset, float] = UNSET
    average_retriever_attribution: Union[None, Unset, float] = UNSET
    average_retriever_utilization: Union[None, Unset, float] = UNSET
    average_sexist: Union[None, Unset, float] = UNSET
    average_tool_error_rate: Union[None, Unset, float] = UNSET
    average_tool_selection_quality: Union[None, Unset, float] = UNSET
    average_total_token_length: Union[None, Unset, float] = UNSET
    average_toxicity: Union[None, Unset, float] = UNSET
    failures_count: Union[None, Unset, int] = 0
    requests_count: Union[None, Unset, int] = 0
    total_cost: Union[None, Unset, float] = 0.0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        average_agentic_workflow_success: Union[None, Unset, float]
        if isinstance(self.average_agentic_workflow_success, Unset):
            average_agentic_workflow_success = UNSET
        else:
            average_agentic_workflow_success = self.average_agentic_workflow_success

        average_completeness_gpt: Union[None, Unset, float]
        if isinstance(self.average_completeness_gpt, Unset):
            average_completeness_gpt = UNSET
        else:
            average_completeness_gpt = self.average_completeness_gpt

        average_factuality: Union[None, Unset, float]
        if isinstance(self.average_factuality, Unset):
            average_factuality = UNSET
        else:
            average_factuality = self.average_factuality

        average_groundedness: Union[None, Unset, float]
        if isinstance(self.average_groundedness, Unset):
            average_groundedness = UNSET
        else:
            average_groundedness = self.average_groundedness

        average_input_sexist: Union[None, Unset, float]
        if isinstance(self.average_input_sexist, Unset):
            average_input_sexist = UNSET
        else:
            average_input_sexist = self.average_input_sexist

        average_input_token_length: Union[None, Unset, float]
        if isinstance(self.average_input_token_length, Unset):
            average_input_token_length = UNSET
        else:
            average_input_token_length = self.average_input_token_length

        average_input_toxicity: Union[None, Unset, float]
        if isinstance(self.average_input_toxicity, Unset):
            average_input_toxicity = UNSET
        else:
            average_input_toxicity = self.average_input_toxicity

        average_instruction_adherence: Union[None, Unset, float]
        if isinstance(self.average_instruction_adherence, Unset):
            average_instruction_adherence = UNSET
        else:
            average_instruction_adherence = self.average_instruction_adherence

        average_latency: Union[None, Unset, float]
        if isinstance(self.average_latency, Unset):
            average_latency = UNSET
        else:
            average_latency = self.average_latency

        average_output_token_length: Union[None, Unset, float]
        if isinstance(self.average_output_token_length, Unset):
            average_output_token_length = UNSET
        else:
            average_output_token_length = self.average_output_token_length

        average_prompt_perplexity: Union[None, Unset, float]
        if isinstance(self.average_prompt_perplexity, Unset):
            average_prompt_perplexity = UNSET
        else:
            average_prompt_perplexity = self.average_prompt_perplexity

        average_retriever_attribution: Union[None, Unset, float]
        if isinstance(self.average_retriever_attribution, Unset):
            average_retriever_attribution = UNSET
        else:
            average_retriever_attribution = self.average_retriever_attribution

        average_retriever_utilization: Union[None, Unset, float]
        if isinstance(self.average_retriever_utilization, Unset):
            average_retriever_utilization = UNSET
        else:
            average_retriever_utilization = self.average_retriever_utilization

        average_sexist: Union[None, Unset, float]
        if isinstance(self.average_sexist, Unset):
            average_sexist = UNSET
        else:
            average_sexist = self.average_sexist

        average_tool_error_rate: Union[None, Unset, float]
        if isinstance(self.average_tool_error_rate, Unset):
            average_tool_error_rate = UNSET
        else:
            average_tool_error_rate = self.average_tool_error_rate

        average_tool_selection_quality: Union[None, Unset, float]
        if isinstance(self.average_tool_selection_quality, Unset):
            average_tool_selection_quality = UNSET
        else:
            average_tool_selection_quality = self.average_tool_selection_quality

        average_total_token_length: Union[None, Unset, float]
        if isinstance(self.average_total_token_length, Unset):
            average_total_token_length = UNSET
        else:
            average_total_token_length = self.average_total_token_length

        average_toxicity: Union[None, Unset, float]
        if isinstance(self.average_toxicity, Unset):
            average_toxicity = UNSET
        else:
            average_toxicity = self.average_toxicity

        failures_count: Union[None, Unset, int]
        if isinstance(self.failures_count, Unset):
            failures_count = UNSET
        else:
            failures_count = self.failures_count

        requests_count: Union[None, Unset, int]
        if isinstance(self.requests_count, Unset):
            requests_count = UNSET
        else:
            requests_count = self.requests_count

        total_cost: Union[None, Unset, float]
        if isinstance(self.total_cost, Unset):
            total_cost = UNSET
        else:
            total_cost = self.total_cost

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if average_agentic_workflow_success is not UNSET:
            field_dict["average_agentic_workflow_success"] = average_agentic_workflow_success
        if average_completeness_gpt is not UNSET:
            field_dict["average_completeness_gpt"] = average_completeness_gpt
        if average_factuality is not UNSET:
            field_dict["average_factuality"] = average_factuality
        if average_groundedness is not UNSET:
            field_dict["average_groundedness"] = average_groundedness
        if average_input_sexist is not UNSET:
            field_dict["average_input_sexist"] = average_input_sexist
        if average_input_token_length is not UNSET:
            field_dict["average_input_token_length"] = average_input_token_length
        if average_input_toxicity is not UNSET:
            field_dict["average_input_toxicity"] = average_input_toxicity
        if average_instruction_adherence is not UNSET:
            field_dict["average_instruction_adherence"] = average_instruction_adherence
        if average_latency is not UNSET:
            field_dict["average_latency"] = average_latency
        if average_output_token_length is not UNSET:
            field_dict["average_output_token_length"] = average_output_token_length
        if average_prompt_perplexity is not UNSET:
            field_dict["average_prompt_perplexity"] = average_prompt_perplexity
        if average_retriever_attribution is not UNSET:
            field_dict["average_retriever_attribution"] = average_retriever_attribution
        if average_retriever_utilization is not UNSET:
            field_dict["average_retriever_utilization"] = average_retriever_utilization
        if average_sexist is not UNSET:
            field_dict["average_sexist"] = average_sexist
        if average_tool_error_rate is not UNSET:
            field_dict["average_tool_error_rate"] = average_tool_error_rate
        if average_tool_selection_quality is not UNSET:
            field_dict["average_tool_selection_quality"] = average_tool_selection_quality
        if average_total_token_length is not UNSET:
            field_dict["average_total_token_length"] = average_total_token_length
        if average_toxicity is not UNSET:
            field_dict["average_toxicity"] = average_toxicity
        if failures_count is not UNSET:
            field_dict["failures_count"] = failures_count
        if requests_count is not UNSET:
            field_dict["requests_count"] = requests_count
        if total_cost is not UNSET:
            field_dict["total_cost"] = total_cost

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_average_agentic_workflow_success(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_agentic_workflow_success = _parse_average_agentic_workflow_success(
            d.pop("average_agentic_workflow_success", UNSET)
        )

        def _parse_average_completeness_gpt(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_completeness_gpt = _parse_average_completeness_gpt(d.pop("average_completeness_gpt", UNSET))

        def _parse_average_factuality(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_factuality = _parse_average_factuality(d.pop("average_factuality", UNSET))

        def _parse_average_groundedness(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_groundedness = _parse_average_groundedness(d.pop("average_groundedness", UNSET))

        def _parse_average_input_sexist(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_input_sexist = _parse_average_input_sexist(d.pop("average_input_sexist", UNSET))

        def _parse_average_input_token_length(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_input_token_length = _parse_average_input_token_length(d.pop("average_input_token_length", UNSET))

        def _parse_average_input_toxicity(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_input_toxicity = _parse_average_input_toxicity(d.pop("average_input_toxicity", UNSET))

        def _parse_average_instruction_adherence(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_instruction_adherence = _parse_average_instruction_adherence(
            d.pop("average_instruction_adherence", UNSET)
        )

        def _parse_average_latency(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_latency = _parse_average_latency(d.pop("average_latency", UNSET))

        def _parse_average_output_token_length(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_output_token_length = _parse_average_output_token_length(d.pop("average_output_token_length", UNSET))

        def _parse_average_prompt_perplexity(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_prompt_perplexity = _parse_average_prompt_perplexity(d.pop("average_prompt_perplexity", UNSET))

        def _parse_average_retriever_attribution(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_retriever_attribution = _parse_average_retriever_attribution(
            d.pop("average_retriever_attribution", UNSET)
        )

        def _parse_average_retriever_utilization(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_retriever_utilization = _parse_average_retriever_utilization(
            d.pop("average_retriever_utilization", UNSET)
        )

        def _parse_average_sexist(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_sexist = _parse_average_sexist(d.pop("average_sexist", UNSET))

        def _parse_average_tool_error_rate(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_tool_error_rate = _parse_average_tool_error_rate(d.pop("average_tool_error_rate", UNSET))

        def _parse_average_tool_selection_quality(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_tool_selection_quality = _parse_average_tool_selection_quality(
            d.pop("average_tool_selection_quality", UNSET)
        )

        def _parse_average_total_token_length(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_total_token_length = _parse_average_total_token_length(d.pop("average_total_token_length", UNSET))

        def _parse_average_toxicity(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_toxicity = _parse_average_toxicity(d.pop("average_toxicity", UNSET))

        def _parse_failures_count(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        failures_count = _parse_failures_count(d.pop("failures_count", UNSET))

        def _parse_requests_count(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        requests_count = _parse_requests_count(d.pop("requests_count", UNSET))

        def _parse_total_cost(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        total_cost = _parse_total_cost(d.pop("total_cost", UNSET))

        aggregate_metrics = cls(
            average_agentic_workflow_success=average_agentic_workflow_success,
            average_completeness_gpt=average_completeness_gpt,
            average_factuality=average_factuality,
            average_groundedness=average_groundedness,
            average_input_sexist=average_input_sexist,
            average_input_token_length=average_input_token_length,
            average_input_toxicity=average_input_toxicity,
            average_instruction_adherence=average_instruction_adherence,
            average_latency=average_latency,
            average_output_token_length=average_output_token_length,
            average_prompt_perplexity=average_prompt_perplexity,
            average_retriever_attribution=average_retriever_attribution,
            average_retriever_utilization=average_retriever_utilization,
            average_sexist=average_sexist,
            average_tool_error_rate=average_tool_error_rate,
            average_tool_selection_quality=average_tool_selection_quality,
            average_total_token_length=average_total_token_length,
            average_toxicity=average_toxicity,
            failures_count=failures_count,
            requests_count=requests_count,
            total_cost=total_cost,
        )

        aggregate_metrics.additional_properties = d
        return aggregate_metrics

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Literal, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CategoricalPromptFilterParam")


@_attrs_define
class CategoricalPromptFilterParam:
    """
    Attributes:
        categories (list[str]):
        column (str):
        filter_type (Literal['category']):
        operator (Union[Unset, Any]): Operator to use when checking if the value is in the categories. If None, we
            default to 'or'. Default: 'any'.
    """

    categories: list[str]
    column: str
    filter_type: Literal["category"]
    operator: Union[Unset, Any] = "any"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        categories = self.categories

        column = self.column

        filter_type = self.filter_type

        operator = self.operator

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"categories": categories, "column": column, "filter_type": filter_type})
        if operator is not UNSET:
            field_dict["operator"] = operator

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        categories = cast(list[str], d.pop("categories"))

        column = d.pop("column")

        filter_type = cast(Literal["category"], d.pop("filter_type"))
        if filter_type != "category":
            raise ValueError(f"filter_type must match const 'category', got '{filter_type}'")

        operator = d.pop("operator", UNSET)

        categorical_prompt_filter_param = cls(
            categories=categories, column=column, filter_type=filter_type, operator=operator
        )

        categorical_prompt_filter_param.additional_properties = d
        return categorical_prompt_filter_param

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from galileo.base import BaseClientModel
from galileo.resources.api.data import list_scorers_with_filters_scorers_list_post
from galileo.resources.api.run_scorer_settings import (
    upsert_scorers_config_projects_project_id_runs_run_id_scorer_settings_post,
)
from galileo.resources.models import (
    ListScorersRequest,
    RunScorerSettingsRequest,
    ScorerTypeFilter,
    ScorerTypeFilterOperator,
    ScorerTypes,
)


class Scorers(BaseClientModel):
    def list(self):
        body = ListScorersRequest(
            filters=[ScorerTypeFilter(value=ScorerTypes.PRESET, operator=ScorerTypeFilterOperator.EQ)]
        )
        result = list_scorers_with_filters_scorers_list_post.sync(client=self.client, body=body)
        return result.scorers


class ScorerSettings(BaseClientModel):
    def create(self, project_id, run_id, scorers):
        return upsert_scorers_config_projects_project_id_runs_run_id_scorer_settings_post.sync(
            project_id=project_id,
            run_id=run_id,
            client=self.client,
            body=RunScorerSettingsRequest(run_id=run_id, scorers=scorers),
        )



from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.aws_credential_type import AwsCredentialType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.aws_sage_maker_integration_create_token import AwsSageMakerIntegrationCreateToken
    from ..models.model import Model


T = TypeVar("T", bound="AwsSageMakerIntegrationCreate")


@_attrs_define
class AwsSageMakerIntegrationCreate:
    """
    Attributes:
        token (AwsSageMakerIntegrationCreateToken):
        credential_type (Union[Unset, AwsCredentialType]):
        models (Union[Unset, list['Model']]):
        region (Union[Unset, str]):  Default: 'us-west-2'.
    """

    token: "AwsSageMakerIntegrationCreateToken"
    credential_type: Union[Unset, AwsCredentialType] = UNSET
    models: Union[Unset, list["Model"]] = UNSET
    region: Union[Unset, str] = "us-west-2"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        token = self.token.to_dict()

        credential_type: Union[Unset, str] = UNSET
        if not isinstance(self.credential_type, Unset):
            credential_type = self.credential_type.value

        models: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.models, Unset):
            models = []
            for models_item_data in self.models:
                models_item = models_item_data.to_dict()
                models.append(models_item)

        region = self.region

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"token": token})
        if credential_type is not UNSET:
            field_dict["credential_type"] = credential_type
        if models is not UNSET:
            field_dict["models"] = models
        if region is not UNSET:
            field_dict["region"] = region

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.aws_sage_maker_integration_create_token import AwsSageMakerIntegrationCreateToken
        from ..models.model import Model

        d = src_dict.copy()
        token = AwsSageMakerIntegrationCreateToken.from_dict(d.pop("token"))

        _credential_type = d.pop("credential_type", UNSET)
        credential_type: Union[Unset, AwsCredentialType]
        if isinstance(_credential_type, Unset):
            credential_type = UNSET
        else:
            credential_type = AwsCredentialType(_credential_type)

        models = []
        _models = d.pop("models", UNSET)
        for models_item_data in _models or []:
            models_item = Model.from_dict(models_item_data)

            models.append(models_item)

        region = d.pop("region", UNSET)

        aws_sage_maker_integration_create = cls(
            token=token, credential_type=credential_type, models=models, region=region
        )

        aws_sage_maker_integration_create.additional_properties = d
        return aws_sage_maker_integration_create

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.split import Split
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.filter_params import FilterParams
    from ..models.multi_label_task_metrics import MultiLabelTaskMetrics


T = TypeVar("T", bound="MetricsResponse")


@_attrs_define
class MetricsResponse:
    """
    Attributes:
        accuracy (Union[None, Unset, float]):
        compare_to (Union[None, Split, Unset]):
        confidence (Union[None, Unset, float]):
        data_error_potential (Union[None, Unset, float]):
        f1 (Union[None, Unset, float]):
        filter_params (Union[Unset, FilterParams]):
        map_threshold (Union[Unset, float]):  Default: 0.5.
        meta_cols (Union[None, Unset, list[str]]):
        multi_label_task_metrics (Union[Unset, MultiLabelTaskMetrics]): Metrics per task for multi-label models.
        precision (Union[None, Unset, float]):
        recall (Union[None, Unset, float]):
        task (Union[None, Unset, str]):
    """

    accuracy: Union[None, Unset, float] = UNSET
    compare_to: Union[None, Split, Unset] = UNSET
    confidence: Union[None, Unset, float] = UNSET
    data_error_potential: Union[None, Unset, float] = UNSET
    f1: Union[None, Unset, float] = UNSET
    filter_params: Union[Unset, "FilterParams"] = UNSET
    map_threshold: Union[Unset, float] = 0.5
    meta_cols: Union[None, Unset, list[str]] = UNSET
    multi_label_task_metrics: Union[Unset, "MultiLabelTaskMetrics"] = UNSET
    precision: Union[None, Unset, float] = UNSET
    recall: Union[None, Unset, float] = UNSET
    task: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        accuracy: Union[None, Unset, float]
        if isinstance(self.accuracy, Unset):
            accuracy = UNSET
        else:
            accuracy = self.accuracy

        compare_to: Union[None, Unset, str]
        if isinstance(self.compare_to, Unset):
            compare_to = UNSET
        elif isinstance(self.compare_to, Split):
            compare_to = self.compare_to.value
        else:
            compare_to = self.compare_to

        confidence: Union[None, Unset, float]
        if isinstance(self.confidence, Unset):
            confidence = UNSET
        else:
            confidence = self.confidence

        data_error_potential: Union[None, Unset, float]
        if isinstance(self.data_error_potential, Unset):
            data_error_potential = UNSET
        else:
            data_error_potential = self.data_error_potential

        f1: Union[None, Unset, float]
        if isinstance(self.f1, Unset):
            f1 = UNSET
        else:
            f1 = self.f1

        filter_params: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.filter_params, Unset):
            filter_params = self.filter_params.to_dict()

        map_threshold = self.map_threshold

        meta_cols: Union[None, Unset, list[str]]
        if isinstance(self.meta_cols, Unset):
            meta_cols = UNSET
        elif isinstance(self.meta_cols, list):
            meta_cols = self.meta_cols

        else:
            meta_cols = self.meta_cols

        multi_label_task_metrics: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.multi_label_task_metrics, Unset):
            multi_label_task_metrics = self.multi_label_task_metrics.to_dict()

        precision: Union[None, Unset, float]
        if isinstance(self.precision, Unset):
            precision = UNSET
        else:
            precision = self.precision

        recall: Union[None, Unset, float]
        if isinstance(self.recall, Unset):
            recall = UNSET
        else:
            recall = self.recall

        task: Union[None, Unset, str]
        if isinstance(self.task, Unset):
            task = UNSET
        else:
            task = self.task

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if accuracy is not UNSET:
            field_dict["accuracy"] = accuracy
        if compare_to is not UNSET:
            field_dict["compare_to"] = compare_to
        if confidence is not UNSET:
            field_dict["confidence"] = confidence
        if data_error_potential is not UNSET:
            field_dict["data_error_potential"] = data_error_potential
        if f1 is not UNSET:
            field_dict["f1"] = f1
        if filter_params is not UNSET:
            field_dict["filter_params"] = filter_params
        if map_threshold is not UNSET:
            field_dict["map_threshold"] = map_threshold
        if meta_cols is not UNSET:
            field_dict["meta_cols"] = meta_cols
        if multi_label_task_metrics is not UNSET:
            field_dict["multi_label_task_metrics"] = multi_label_task_metrics
        if precision is not UNSET:
            field_dict["precision"] = precision
        if recall is not UNSET:
            field_dict["recall"] = recall
        if task is not UNSET:
            field_dict["task"] = task

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.filter_params import FilterParams
        from ..models.multi_label_task_metrics import MultiLabelTaskMetrics

        d = src_dict.copy()

        def _parse_accuracy(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        accuracy = _parse_accuracy(d.pop("accuracy", UNSET))

        def _parse_compare_to(data: object) -> Union[None, Split, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                compare_to_type_0 = Split(data)

                return compare_to_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Split, Unset], data)

        compare_to = _parse_compare_to(d.pop("compare_to", UNSET))

        def _parse_confidence(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        confidence = _parse_confidence(d.pop("confidence", UNSET))

        def _parse_data_error_potential(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        data_error_potential = _parse_data_error_potential(d.pop("data_error_potential", UNSET))

        def _parse_f1(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        f1 = _parse_f1(d.pop("f1", UNSET))

        _filter_params = d.pop("filter_params", UNSET)
        filter_params: Union[Unset, FilterParams]
        if isinstance(_filter_params, Unset):
            filter_params = UNSET
        else:
            filter_params = FilterParams.from_dict(_filter_params)

        map_threshold = d.pop("map_threshold", UNSET)

        def _parse_meta_cols(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                meta_cols_type_0 = cast(list[str], data)

                return meta_cols_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        meta_cols = _parse_meta_cols(d.pop("meta_cols", UNSET))

        _multi_label_task_metrics = d.pop("multi_label_task_metrics", UNSET)
        multi_label_task_metrics: Union[Unset, MultiLabelTaskMetrics]
        if isinstance(_multi_label_task_metrics, Unset):
            multi_label_task_metrics = UNSET
        else:
            multi_label_task_metrics = MultiLabelTaskMetrics.from_dict(_multi_label_task_metrics)

        def _parse_precision(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        precision = _parse_precision(d.pop("precision", UNSET))

        def _parse_recall(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        recall = _parse_recall(d.pop("recall", UNSET))

        def _parse_task(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        task = _parse_task(d.pop("task", UNSET))

        metrics_response = cls(
            accuracy=accuracy,
            compare_to=compare_to,
            confidence=confidence,
            data_error_potential=data_error_potential,
            f1=f1,
            filter_params=filter_params,
            map_threshold=map_threshold,
            meta_cols=meta_cols,
            multi_label_task_metrics=multi_label_task_metrics,
            precision=precision,
            recall=recall,
            task=task,
        )

        metrics_response.additional_properties = d
        return metrics_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="UserPasswordResetRequest")


@_attrs_define
class UserPasswordResetRequest:
    """
    Attributes:
        email (str):
        email_is_verified (Union[Unset, bool]):  Default: True.
        password (Union[None, Unset, str]):
    """

    email: str
    email_is_verified: Union[Unset, bool] = True
    password: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        email = self.email

        email_is_verified = self.email_is_verified

        password: Union[None, Unset, str]
        if isinstance(self.password, Unset):
            password = UNSET
        else:
            password = self.password

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"email": email})
        if email_is_verified is not UNSET:
            field_dict["email_is_verified"] = email_is_verified
        if password is not UNSET:
            field_dict["password"] = password

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        email = d.pop("email")

        email_is_verified = d.pop("email_is_verified", UNSET)

        def _parse_password(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        password = _parse_password(d.pop("password", UNSET))

        user_password_reset_request = cls(email=email, email_is_verified=email_is_verified, password=password)

        user_password_reset_request.additional_properties = d
        return user_password_reset_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.metric_computation_status import MetricComputationStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.metric_computation_value_type_4 import MetricComputationValueType4


T = TypeVar("T", bound="MetricComputation")


@_attrs_define
class MetricComputation:
    """
    Attributes:
        error_message (Union[None, Unset, str]):
        execution_time (Union[None, Unset, float]):
        status (Union[MetricComputationStatus, None, Unset]):
        value (Union['MetricComputationValueType4', None, Unset, float, int, list[Union[None, float, int, str]], str]):
    """

    error_message: Union[None, Unset, str] = UNSET
    execution_time: Union[None, Unset, float] = UNSET
    status: Union[MetricComputationStatus, None, Unset] = UNSET
    value: Union["MetricComputationValueType4", None, Unset, float, int, list[Union[None, float, int, str]], str] = (
        UNSET
    )
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.metric_computation_value_type_4 import MetricComputationValueType4

        error_message: Union[None, Unset, str]
        if isinstance(self.error_message, Unset):
            error_message = UNSET
        else:
            error_message = self.error_message

        execution_time: Union[None, Unset, float]
        if isinstance(self.execution_time, Unset):
            execution_time = UNSET
        else:
            execution_time = self.execution_time

        status: Union[None, Unset, str]
        if isinstance(self.status, Unset):
            status = UNSET
        elif isinstance(self.status, MetricComputationStatus):
            status = self.status.value
        else:
            status = self.status

        value: Union[None, Unset, dict[str, Any], float, int, list[Union[None, float, int, str]], str]
        if isinstance(self.value, Unset):
            value = UNSET
        elif isinstance(self.value, list):
            value = []
            for value_type_3_item_data in self.value:
                value_type_3_item: Union[None, float, int, str]
                value_type_3_item = value_type_3_item_data
                value.append(value_type_3_item)

        elif isinstance(self.value, MetricComputationValueType4):
            value = self.value.to_dict()
        else:
            value = self.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if error_message is not UNSET:
            field_dict["error_message"] = error_message
        if execution_time is not UNSET:
            field_dict["execution_time"] = execution_time
        if status is not UNSET:
            field_dict["status"] = status
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.metric_computation_value_type_4 import MetricComputationValueType4

        d = src_dict.copy()

        def _parse_error_message(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        error_message = _parse_error_message(d.pop("error_message", UNSET))

        def _parse_execution_time(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        execution_time = _parse_execution_time(d.pop("execution_time", UNSET))

        def _parse_status(data: object) -> Union[MetricComputationStatus, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                status_type_0 = MetricComputationStatus(data)

                return status_type_0
            except:  # noqa: E722
                pass
            return cast(Union[MetricComputationStatus, None, Unset], data)

        status = _parse_status(d.pop("status", UNSET))

        def _parse_value(
            data: object,
        ) -> Union["MetricComputationValueType4", None, Unset, float, int, list[Union[None, float, int, str]], str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                value_type_3 = []
                _value_type_3 = data
                for value_type_3_item_data in _value_type_3:

                    def _parse_value_type_3_item(data: object) -> Union[None, float, int, str]:
                        if data is None:
                            return data
                        return cast(Union[None, float, int, str], data)

                    value_type_3_item = _parse_value_type_3_item(value_type_3_item_data)

                    value_type_3.append(value_type_3_item)

                return value_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                value_type_4 = MetricComputationValueType4.from_dict(data)

                return value_type_4
            except:  # noqa: E722
                pass
            return cast(
                Union["MetricComputationValueType4", None, Unset, float, int, list[Union[None, float, int, str]], str],
                data,
            )

        value = _parse_value(d.pop("value", UNSET))

        metric_computation = cls(error_message=error_message, execution_time=execution_time, status=status, value=value)

        metric_computation.additional_properties = d
        return metric_computation

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="RangePromptFilterParam")


@_attrs_define
class RangePromptFilterParam:
    """
    Attributes:
        column (str):
        filter_type (Literal['range']):
        high (float):
        low (float):
    """

    column: str
    filter_type: Literal["range"]
    high: float
    low: float
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        column = self.column

        filter_type = self.filter_type

        high = self.high

        low = self.low

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"column": column, "filter_type": filter_type, "high": high, "low": low})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        column = d.pop("column")

        filter_type = cast(Literal["range"], d.pop("filter_type"))
        if filter_type != "range":
            raise ValueError(f"filter_type must match const 'range', got '{filter_type}'")

        high = d.pop("high")

        low = d.pop("low")

        range_prompt_filter_param = cls(column=column, filter_type=filter_type, high=high, low=low)

        range_prompt_filter_param.additional_properties = d
        return range_prompt_filter_param

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.job_status import JobStatus
from ..models.scorer_type import ScorerType
from ..types import UNSET, Unset

T = TypeVar("T", bound="JobInfo")


@_attrs_define
class JobInfo:
    """
    Attributes:
        job_error_message (Union[None, Unset, str]): Error message to show the users on hover in case the job fails or
            errors out.
        job_progress_message (Union[None, Unset, str]): Progress message to show the users on hover in case the job is
            in progress.
        job_status (Union[JobStatus, None, Unset]): Job status used for computing the column. Only set for metrics
            columns that have a separate scorer job.
        job_type (Union[None, ScorerType, Unset]): Whether a job is designated as plus or basic.
        scorer_name (Union[None, Unset, str]): Scorer Name executed by the job.
    """

    job_error_message: Union[None, Unset, str] = UNSET
    job_progress_message: Union[None, Unset, str] = UNSET
    job_status: Union[JobStatus, None, Unset] = UNSET
    job_type: Union[None, ScorerType, Unset] = UNSET
    scorer_name: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        job_error_message: Union[None, Unset, str]
        if isinstance(self.job_error_message, Unset):
            job_error_message = UNSET
        else:
            job_error_message = self.job_error_message

        job_progress_message: Union[None, Unset, str]
        if isinstance(self.job_progress_message, Unset):
            job_progress_message = UNSET
        else:
            job_progress_message = self.job_progress_message

        job_status: Union[None, Unset, str]
        if isinstance(self.job_status, Unset):
            job_status = UNSET
        elif isinstance(self.job_status, JobStatus):
            job_status = self.job_status.value
        else:
            job_status = self.job_status

        job_type: Union[None, Unset, str]
        if isinstance(self.job_type, Unset):
            job_type = UNSET
        elif isinstance(self.job_type, ScorerType):
            job_type = self.job_type.value
        else:
            job_type = self.job_type

        scorer_name: Union[None, Unset, str]
        if isinstance(self.scorer_name, Unset):
            scorer_name = UNSET
        else:
            scorer_name = self.scorer_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if job_error_message is not UNSET:
            field_dict["job_error_message"] = job_error_message
        if job_progress_message is not UNSET:
            field_dict["job_progress_message"] = job_progress_message
        if job_status is not UNSET:
            field_dict["job_status"] = job_status
        if job_type is not UNSET:
            field_dict["job_type"] = job_type
        if scorer_name is not UNSET:
            field_dict["scorer_name"] = scorer_name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_job_error_message(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        job_error_message = _parse_job_error_message(d.pop("job_error_message", UNSET))

        def _parse_job_progress_message(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        job_progress_message = _parse_job_progress_message(d.pop("job_progress_message", UNSET))

        def _parse_job_status(data: object) -> Union[JobStatus, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                job_status_type_0 = JobStatus(data)

                return job_status_type_0
            except:  # noqa: E722
                pass
            return cast(Union[JobStatus, None, Unset], data)

        job_status = _parse_job_status(d.pop("job_status", UNSET))

        def _parse_job_type(data: object) -> Union[None, ScorerType, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                job_type_type_0 = ScorerType(data)

                return job_type_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, ScorerType, Unset], data)

        job_type = _parse_job_type(d.pop("job_type", UNSET))

        def _parse_scorer_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        scorer_name = _parse_scorer_name(d.pop("scorer_name", UNSET))

        job_info = cls(
            job_error_message=job_error_message,
            job_progress_message=job_progress_message,
            job_status=job_status,
            job_type=job_type,
            scorer_name=scorer_name,
        )

        job_info.additional_properties = d
        return job_info

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
